package asd_library.tree.bstree;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.*;
import java.util.Random;
import asd_library.BaseObject;

/**
 * This class reads strings and numbers from a generic input stream.
 * Creates a buffered input reader, catches I/O exceptions and converts strings
 * into numbers.
 */
class ConsoleReader
{  /**
    * Creates a console reader on an input stream,
    * as System.in
    * @param inStream un flusso di input
    */
   public ConsoleReader(InputStream inStream)
   {  reader = new BufferedReader
         (new InputStreamReader(inStream));
   }

   /**
      Reads an input line and converts it in a e la converte in un integer.
      La riga di input non pu contenere altro che un integer.
      Non sono neppure ammessi spazi vuoti.
      @return l'integer digitato dall'utente
   */
   public int readInt()
   {  String inputString = readLine();
      int n = Integer.parseInt(inputString);
      return n;
   }

   /**
      Legge una riga di input e la converte in un numero in virgola
      Mobile. La riga di input non pu contenere altro che un
      numero. Non sono neppure ammessi spazi vuoti.
      @return il numero digitato dall'utente
   */
   public double readDouble()
   {  String inputString = readLine();
      double x = Double.parseDouble(inputString);
      return x;
   }

   /**
      Legge una riga di input. Nel caso (improbabile) di
      una eccezione I/O, il programma si arresta.
      @return la riga di input digitata dall'utente, null
      alla fine dell'input
   */
   public String readLine()
   {  String inputLine = "";

      try
      {  inputLine = reader.readLine();
      }
      catch(IOException e)
      {  System.out.println(e);
         System.exit(1);
      }

      return inputLine;
   }

   private BufferedReader reader;
}



/**
 * Test Class of BSTree
 *
 */

public class BSTtreeDemo {
  public final static int MAXVAL = 2000000000;
        private static ConsoleReader lettore = new ConsoleReader(System.in);
        private static Random generatore = new Random();
        public static void main(String args[]) {
                System.out.println("Starting BstTest...");
                BSTree albero = new BSTree();
                int scelta;
                do {
                        menu();
                        scelta = lettore.readInt();
                        lancia(scelta, albero);
                } while(scelta != 0);
                System.out.println("Ciao!!");
        }

        private static void menu() {
                System.out.println(" [0] Uscita");
                System.out.println(" [1] Inserisci elementi random");
                System.out.println(" [2] Inserisci un elemento");
                System.out.println(" [3] Elimina un elemento");
                System.out.println(" [4] Cerca (ricorsivamente) un elemento");
                System.out.println(" [5] Cerca (iterativamente) un elemento");
                System.out.println(" [6] Visita anticipata (ricorsiva)");
                System.out.println(" [7] Visita anticipata (iterativa)");
                System.out.println(" [8] Visita simmetrica (ricorsiva)");
                System.out.println(" [9] Visita simmetrica (iterativa)");
                System.out.println("[10] Visita posticipata (ricorsiva)");
                System.out.println("[11] Visita posticipata (iterativa)");
                System.out.println("[12] Visita BFS (con coda)");
                System.out.println("[13] Numero nodi nell'albero");
                System.out.println("[14] Altezza albero");
                System.out.println("[15] Azzera albero");
        }

        private static void lancia(int s, BSTree a) {
                BaseObject bo;
                BSTNode node;
                switch(s) {
                        case 1: // Inserisci elementi random
                                System.out.print("Quanti elementi? ");
                                int n = lettore.readInt();
                                int prevsize = a.getSize();
                                for(int i = 1; i <= n; i++) {
                                        bo = new BaseObject(generatore.nextInt(MAXVAL));
                                        if(a.iterativeSearch(bo) == null)
                                                a.insert(bo);
                                }
                                System.out.println("Fatto. Inseriti " + (a.getSize() - prevsize) + " elementi");
                                break;
                        case 2: // Inserisci un elemento
                                System.out.print("Elemento? ");
                                bo = new BaseObject(lettore.readInt());
                                if(a.iterativeSearch(bo) == null)
                                        a.insert(bo);
                                else
                                        System.out.println(bo.toString() + " presente");
                                System.out.println("Fatto.");
                                break;
                        case 3: // Inserisci un elemento
                               System.out.print("Elemento? ");
                               bo = new BaseObject(lettore.readInt());
                               a.remove(bo);
                               System.out.println("Fatto.");
                               break;

                        case 4: // Cerca (ricorsivamente) un elemento
                                System.out.print("Elemento? ");
                                node = a.search(new BaseObject(lettore.readInt()));
                                if(node != null)
                                        System.out.println("Tovato " + node.getKey().toString());
                                else
                                        System.out.println("Assente!");
                                break;
                        case 5: // Cerca (iterativamente) un elemento
                                System.out.print("Elemento? ");
                                node = a.iterativeSearch(new BaseObject(lettore.readInt()));
                                if(node != null)
                                        System.out.println("Tovato " + node.getKey().toString());
                                else
                                        System.out.println("Assente!");
                                break;
                        case 6: // Visita anticipata (ricorsiva)
                                System.out.println("preorder...");
                                a.preorder();
                                System.out.println("Fatto.");
                                break;
                        case 7: // Visita anticipata (iterativa)
                                System.out.println("preorder (iterativa)...");
                                a.iterativePreorder();
                                System.out.println("Fatto.");
                                break;
                        case 8: // Visita simmetrica (ricorsiva)
                                System.out.println("inorder...");
                                a.inorder();
                                System.out.println("Fatto.");
                                break;
                        case 9: // Visita simmetrica (iterativa)
                                System.out.println("inorder (iterativa)...");
                                a.iterativeInorder();
                                System.out.println("Fatto.");
                                break;
                        case 10: // Visita posticipata (ricorsiva)
                                System.out.println("postorder...");
                                a.postorder();
                                System.out.println("Fatto.");
                                break;
                        case 11: // Visita posticipata (iterativa)
                                System.out.println("postorder (iterativa)...");
                                a.iterativePostorder();
                                System.out.println("Fatto.");
                                break;
                        case 12: // Visita BFS (con coda)
                                System.out.println("BFS (con coda)...");
                                a.breadthFirst();
                                System.out.println("Fatto.");
                                break;
                        case 13: // Numero nodi nell'albero
                                System.out.println("Nodi: " + a.getSize());
                                break;
                        case 14: // Altezza albero
                                System.out.println("Altezza: " + a.treeHeight());
                                break;
                        case 15: // Azzera albero
                                a.clear();
                                break;
                }
        }

}