package asd_library.tree.heap;

/**
 * The class Heap implements a max-heap
 */

public class Heap implements Heap_adt{
  public static final int DEFAULTCAPACITY = 50;
  private Comparable[] storage;
  private int size;


  public Heap() {
    this(DEFAULTCAPACITY);
  }

  public Heap(int dim) {
    storage = new Comparable[dim];
    size = 0;
  }

/******************************************************************************/
/*                            Public method                                   */
/******************************************************************************/
  /**
   * Returns the max element in this heap.
   * @return max element in this heap.
   * @throws Exception if this heap is empty.
   */
  public Comparable getMax() throws Exception {
    if(!isEmpty())
      return storage[0];
    else
      throw new Exception("getMax requested to empty heap");
  }

  /**
   * Inserts the  specified element key in this heap.
   * @param key element to insert.
   * @throws Exception if this heap is full.
   */
  public void insert(Comparable key) throws Exception{
    if (this.isFull())
      throw new Exception("Full heap");
    else{
      storage[this.size]=key; //put the new key at the end of this heap.
      int index = size++;
      int parent = getParentIndex(index);
      while(!isRoot(index) && (storage[index].compareTo(storage[parent])>0)) {
                exchange(index, parent); // swap celle di storage
                index = parent;
                parent = getParentIndex(index);
      }
    }
  }



  /**
   * Deletes the max element stored in this heap.
   * @throws Exception if this heap is empty.
   */
  public void deleteMax()throws Exception{
    if (this.isEmpty())
      throw new Exception("Empty heap");
    else{
      storage[0] = storage[--size];
      heapify(0);
    }

  }

  /**
   * Exchanges the value stored in the position index e parent in this heap.
   * @param index position of an element in this heap.
   * @param parent position of an element in this heap.
   */
  public void exchange(int index, int parent){
    Comparable temp = this.storage[index];
    this.storage[index]=this.storage[parent];
    this.storage[parent]=temp;

  }


  /**
   * Returns the child with the greatest value of the specified parent.
   * @param parent position of a node
   * @return the child with the greatest value.
   */
  public int getMaxChildIndex(int parent){
    int leftIndex=this.getLeftIndex(parent);
    int rightIndex=this.getRightIndex(parent);
    if(this.storage[leftIndex].compareTo(this.storage[rightIndex])>0){
      return leftIndex;
    }else return rightIndex;
  }

  /**
   * Trickles down the node i until it reaches the position that reestablishes
   * the heap order.
   * @param i node to trickle down.
   */
  public void heapify(int i) {
    if (isLeaf(i))
      return;
    else {
      int j = 0; // inizializzazione fittizia per tacitare compilatore
      try {
        j = getMaxChildIndex(i);
      }
      catch (Exception e) {
        // only if i is a leaf.... already checked
      }
      if (this.storage[i].compareTo(this.storage[j])<0){
        exchange(i, j);
        heapify(j);
      }
    }
  }


  /**
   * Verifies if the specified node i is a leaf.
   * @param i a node.
   * @return true if i is a leaf, false otherwise.
   */
  public boolean isLeaf(int i) {
    return getLeftIndex(i) >= size;
  }

  /**
   * Verifies if the specified node i is the root.
   * @param i a node.
   * @return true if i is the root, false otherwise.
   */
  public boolean isRoot(int i) {
    return i == 0;
  }


  /**
   * Verifies if this heap is empty
   * @return true if this heap is empty, false otherwise.
   */
  public boolean isEmpty() {
    return size == 0;
  }

  /**
   * Verifies if this heap is full
   * @return true if this heap is full, false otherwise.
   */
  public boolean isFull() {
    return size == storage.length;
  }


  /**
   * Returns the number of elements in this heap.
   * @return the number of elements in this heap.
   */
  public int getSize(){
    return this.size;
  }

  private int getLeftIndex(int i) {
    return 2 * i + 1;
  }

  private int getRightIndex(int i) {
    return getLeftIndex(i) + 1;
  }

  private int getParentIndex(int i) {
    return (i - 1) / 2;
  }

  /**
   * Sorting algorithm
   * @param data array to sorting
   */
  public static void heapSort(Comparable[] data) {
    Heap aHeap = array2heap(data);
    for(int i = aHeap.size - 1; i > 0; i--) {
      aHeap.exchange(0, i);
      aHeap.size--;
      aHeap.heapify(0);
    }
    System.arraycopy(aHeap.storage, 0, data, 0, aHeap.storage.length);
  }

  /**
   * Build the heap from the specified array.
   * @param data an array.
   * @return the heap.
   */
  public static Heap array2heap(Comparable[] data) {
    Heap aHeap = new Heap(data.length);
    aHeap.size = data.length;
    System.arraycopy(data, 0, aHeap.storage, 0, data.length);
    for(int i = aHeap.getParentIndex(aHeap.size-1); i >= 0; i--)
      aHeap.heapify(i);
    return aHeap;
  }

  /**
   * Returns a string representation of the heap.
   * @return a <b>String</b> that represents the heap.
    */
  public String toString() {
    StringBuffer aString = new StringBuffer();
    for(int i = 0; i < size; i++)
      aString.append(storage[i]).append(" ");
    aString.append("-- capacity = " + storage.length);
    return aString.toString();
    }




}
