package asd_library.tree.tree;

/**
 * Basic node stored in a binary search tree.
 */

public class TreeNode {

  /**
   * The key of the node.
   */
  protected Comparable key;

  /**
   * The first child node of current node.
   */
  protected TreeNode firstChild;

  /**
   * The next sibling of current node
   */
  protected TreeNode nextSibling;

  /**
   *  The parent of this node
   */
  protected TreeNode parent;

  /**
     * Constructs a new node.
     */
    public TreeNode() {
        parent = firstChild = nextSibling = null;
    }

    /**
     * Constructs a new node.
     * @param el the element to insert.
     */
    public TreeNode(Comparable el) {
        this(el,null,null,null);
    }

    /**
     * Constructs a new node.
     * @param el the element to insert.
     * @param par the parent of this node.
     * @param fc the first child of this node.
     * @param ns the next sibling of this node.
     */
    protected TreeNode(Comparable el,TreeNode par, TreeNode fc, TreeNode ns) {
        key = el; parent = par; firstChild = fc; nextSibling = ns;
    }

    /**
     * Returns the key of the node.
     * @return the key of this node
     */
    public Comparable getKey() {
            return key;
    }

    /**
    * Returns the first child of the node.
    * @return the first child <b>TreeNode</b>.
    */
   public TreeNode getFirstChild() {
           return firstChild;
   }

   /**
    * Returns the next sibling of the node.
    * @return the next sibling <b>TreeNode</b>.
    */
   public TreeNode getNextSibling() {
           return nextSibling;
   }

   /**
    * Returns the parent of the node.
    * @return the parent <b>TreeNode</b>.
    */
   public TreeNode getParent() {
           return parent;
   }

   /**
    * Sets the parent of this node to be the specified node.
    * @param par what the parent field is to be set to.
    */
   public void setParent(TreeNode par){
     this.parent=par;
   }

   /**
    * Prints the <b>String</b> that represents the key of hte node.
    */
   public void visit() {
       System.out.print(key.toString());
   }

   /**
    * Verifies if this <b>TreeNode</b> is a leaf.
    * @return true if this node is a leaf, false otherwise.
    */
   public boolean isLeaf() {
           return (firstChild == null);
   }



}